@@ -41,7 +41,7 @@ PGAPI_BindParameter(
 					SDWORD FAR * pcbValue)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
-	static char *func = "PGAPI_BindParameter";
+	CSTR func = "PGAPI_BindParameter";
 	APDFields	*apdopts;
 	IPDFields	*ipdopts;
 
@@ -141,7 +141,7 @@ PGAPI_BindCol(
 			  SDWORD FAR * pcbValue)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
-	static char *func = "PGAPI_BindCol";
+	CSTR func = "PGAPI_BindCol";
 	ARDFields	*opts;
 
 	mylog("%s: entering...\n", func);
@@ -177,7 +177,6 @@ PGAPI_BindCol(
 		else
 		{
 			/* Make sure it is the bookmark data type */
-			if (fCType == SQL_C_BOOKMARK)
 			switch (fCType)
 			{
 				case SQL_C_BOOKMARK:
@@ -186,14 +185,16 @@ PGAPI_BindCol(
 #endif /* ODBCVER */
 					break;
 				default:
-					SC_set_error(stmt, STMT_PROGRAM_TYPE_OUT_OF_RANGE, "Column 0 is not of type SQL_C_BOOKMARK");
-inolog("Column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
+					SC_set_error(stmt, STMT_PROGRAM_TYPE_OUT_OF_RANGE, "Bind column 0 is not of type SQL_C_BOOKMARK");
+inolog("Bind column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 					SC_log_error(func, "", stmt);
 					return SQL_ERROR;
 			}
 
 			opts->bookmark->buffer = rgbValue;
 			opts->bookmark->used = pcbValue;
+			opts->bookmark->buflen = cbValueMax;
+			opts->bookmark->returntype = fCType;
 		}
 		return SQL_SUCCESS;
 	}
@@ -275,7 +276,7 @@ PGAPI_DescribeParam(
 					SWORD FAR * pfNullable)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
-	static char *func = "PGAPI_DescribeParam";
+	CSTR func = "PGAPI_DescribeParam";
 	APDFields	*apdopts;
 	IPDFields	*ipdopts;
 
@@ -327,7 +328,7 @@ PGAPI_ParamOptions(
 				   UDWORD crow,
 				   UDWORD FAR * pirow)
 {
-	static char *func = "PGAPI_ParamOptions";
+	CSTR func = "PGAPI_ParamOptions";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	APDFields	*apdopts;
 
@@ -357,7 +358,7 @@ PGAPI_NumParams(
 	StatementClass *stmt = (StatementClass *) hstmt;
 	char		in_quote = FALSE;
 	unsigned int i;
-	static char *func = "PGAPI_NumParams";
+	CSTR func = "PGAPI_NumParams";
 
 	mylog("%s: entering...\n", func);
 
@@ -430,7 +431,7 @@ create_empty_bindings(int num_columns)
 void
 extend_parameter_bindings(APDFields *self, int num_params)
 {
-	static char *func = "extend_parameter_bindings";
+	CSTR func = "extend_parameter_bindings";
 	ParameterInfoClass *new_bindings;
 
 	mylog("%s: entering ... self=%u, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
@@ -463,7 +464,7 @@ extend_parameter_bindings(APDFields *self, int num_params)
 void
 extend_iparameter_bindings(IPDFields *self, int num_params)
 {
-	static char *func = "extend_iparameter_bindings";
+	CSTR func = "extend_iparameter_bindings";
 	ParameterImplClass *new_bindings;
 
 	mylog("%s: entering ... self=%u, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
@@ -496,7 +497,7 @@ extend_iparameter_bindings(IPDFields *self, int num_params)
 void
 reset_a_parameter_binding(APDFields *self, int ipar)
 {
-	static char *func = "reset_a_parameter_binding";
+	CSTR func = "reset_a_parameter_binding";
 
 	mylog("%s: entering ... self=%u, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
 
@@ -529,7 +530,7 @@ reset_a_parameter_binding(APDFields *self, int ipar)
 void
 reset_a_iparameter_binding(IPDFields *self, int ipar)
 {
-	static char *func = "reset_a_iparameter_binding";
+	CSTR func = "reset_a_iparameter_binding";
 
 	mylog("%s: entering ... self=%u, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
 
@@ -609,7 +610,7 @@ IPD_free_params(IPDFields *ipdopts, char option)
 void
 extend_column_bindings(ARDFields *self, int num_columns)
 {
-	static char *func = "extend_column_bindings";
+	CSTR func = "extend_column_bindings";
 	BindInfoClass *new_bindings;
 	int			i;
 
@@ -663,7 +664,7 @@ extend_column_bindings(ARDFields *self, int num_columns)
 void
 reset_a_column_binding(ARDFields *self, int icol)
 {
-	static char *func = "reset_a_column_binding";
+	CSTR func = "reset_a_column_binding";
 
 	mylog("%s: entering ... self=%u, bindings_allocated=%d, icol=%d\n", func, self, self->allocated, icol);
 
