@@ -50,7 +50,7 @@ PGAPI_AllocConnect(
 {
 	EnvironmentClass *env = (EnvironmentClass *) henv;
 	ConnectionClass *conn;
-	static char *func = "PGAPI_AllocConnect";
+	CSTR func = "PGAPI_AllocConnect";
 
 	mylog("%s: entering...\n", func);
 
@@ -95,7 +95,7 @@ PGAPI_Connect(
 {
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
 	ConnInfo   *ci;
-	static char *func = "PGAPI_Connect";
+	CSTR func = "PGAPI_Connect";
 
 	mylog("%s: entering...\n", func);
 
@@ -150,11 +150,14 @@ PGAPI_BrowseConnect(
 					SWORD cbConnStrOutMax,
 					SWORD FAR * pcbConnStrOut)
 {
-	static char *func = "PGAPI_BrowseConnect";
+	CSTR func = "PGAPI_BrowseConnect";
+	ConnectionClass *conn = (ConnectionClass *) hdbc;
 
 	mylog("%s: entering...\n", func);
 
-	return SQL_SUCCESS;
+	CC_set_error(conn, CONN_NOT_IMPLEMENTED_ERROR, "not implemented");
+	CC_log_error(func, "Function not implemented", conn);
+	return SQL_ERROR;
 }
 
 
@@ -164,7 +167,7 @@ PGAPI_Disconnect(
 				 HDBC hdbc)
 {
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
-	static char *func = "PGAPI_Disconnect";
+	CSTR func = "PGAPI_Disconnect";
 
 
 	mylog("%s: entering...\n", func);
@@ -202,7 +205,7 @@ PGAPI_FreeConnect(
 				  HDBC hdbc)
 {
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
-	static char *func = "PGAPI_FreeConnect";
+	CSTR func = "PGAPI_FreeConnect";
 
 	mylog("%s: entering...\n", func);
 	mylog("**** in %s: hdbc=%u\n", func, hdbc);
@@ -623,7 +626,7 @@ CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 	int			beresp;
 	char		msgbuffer[ERROR_MSG_LENGTH];
 	char		salt[5], notice[512];
-	static char *func = "CC_connect";
+	CSTR		func = "CC_connect";
 	char	   *encoding;
 
 	mylog("%s: entering...\n", func);
@@ -1193,6 +1196,8 @@ void	CC_on_abort(ConnectionClass *conn, UDWORD opt)
  *	(i.e., C3326857) for SQL select statements.  This cursor is then used in future
  *	'declare cursor C3326857 for ...' and 'fetch 100 in C3326857' statements.
  */
+
+
 QResultClass *
 CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag)
 {
@@ -1249,12 +1254,13 @@ CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag)
 		return NULL;
 	}
 
+	ENTER_CONN_CS(self);
 	SOCK_put_char(sock, 'Q');
 	if (SOCK_get_errcode(sock) != 0)
 	{
 		CC_set_error(self, CONNECTION_COULD_NOT_SEND, "Could not send Query to backend");
 		CC_on_abort(self, NO_TRANS | CONN_DEAD);
-		return NULL;
+		RETURN_AFTER_LEAVE_CS(self, NULL);
 	}
 
 	if (issue_begin)
@@ -1266,7 +1272,7 @@ CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag)
 	{
 		CC_set_error(self, CONNECTION_COULD_NOT_SEND, "Could not send Query to backend");
 		CC_on_abort(self, NO_TRANS | CONN_DEAD);
-		return NULL;
+		RETURN_AFTER_LEAVE_CS(self, NULL);
 	}
 
 	mylog("send_query: done sending query\n");
@@ -1286,7 +1292,7 @@ CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag)
 		if (!cmdres)
 		{
 			CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, "Could not create result info in send_query.");
-			return NULL;
+			RETURN_AFTER_LEAVE_CS(self, NULL);
 		}
 	}
 	res = cmdres;
@@ -1610,7 +1616,7 @@ CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag)
 			}
 		}
 	}
-	return retres;
+	RETURN_AFTER_LEAVE_CS(self, retres);
 }
 
 
@@ -1786,7 +1792,7 @@ CC_send_settings(ConnectionClass *self)
 #ifdef	HAVE_STRTOK_R
 	char	*last;
 #endif /* HAVE_STRTOK_R */
-	static char *func = "CC_send_settings";
+	CSTR func = "CC_send_settings";
 
 
 	mylog("%s: entering...\n", func);
@@ -1915,7 +1921,7 @@ CC_lookup_lo(ConnectionClass *self)
 	HSTMT		hstmt;
 	StatementClass *stmt;
 	RETCODE		result;
-	static char *func = "CC_lookup_lo";
+	CSTR func = "CC_lookup_lo";
 
 	mylog("%s: entering...\n", func);
 
@@ -2000,7 +2006,7 @@ CC_lookup_pg_version(ConnectionClass *self)
 	char		szVersion[32];
 	int			major,
 				minor;
-	static char *func = "CC_lookup_pg_version";
+	CSTR		func = "CC_lookup_pg_version";
 
 	mylog("%s: entering...\n", func);
 
