@@ -36,7 +36,7 @@ char		EN_Destructor(EnvironmentClass *self);
 char		EN_get_error(EnvironmentClass *self, int *number, char **message);
 char		EN_add_connection(EnvironmentClass *self, ConnectionClass *conn);
 char		EN_remove_connection(EnvironmentClass *self, ConnectionClass *conn);
-void		EN_log_error(char *func, char *desc, EnvironmentClass *self);
+void		EN_log_error(const char *func, char *desc, EnvironmentClass *self);
 
 #define	EN_OV_ODBC2	1L
 #define	EN_CONN_POOLING	(1L<<1)
