@@ -36,10 +36,10 @@ PGAPI_Prepare(HSTMT hstmt,
 			  UCHAR FAR * szSqlStr,
 			  SDWORD cbSqlStr)
 {
-	static char *func = "PGAPI_Prepare";
+	CSTR func = "PGAPI_Prepare";
 	StatementClass *self = (StatementClass *) hstmt;
 
-	mylog("%s: entering...\n", func);
+	mylog("%s: entering... len=%d\n", func, cbSqlStr);
 
 	if (!self)
 	{
@@ -134,7 +134,7 @@ PGAPI_ExecDirect(
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
 	RETCODE		result;
-	static char *func = "PGAPI_ExecDirect";
+	CSTR func = "PGAPI_ExecDirect";
 
 	mylog("%s: entering...\n", func);
 
@@ -195,7 +195,7 @@ PGAPI_ExecDirect(
 static
 RETCODE	Exec_with_parameters_resolved(StatementClass *stmt, BOOL *exec_end)
 {
-	static const char *func = "Exec_with_parameters_resolved";
+	CSTR func = "Exec_with_parameters_resolved";
 	RETCODE		retval;
 	int		end_row, cursor_type, scroll_concurrency;
 	ConnectionClass	*conn;
@@ -293,6 +293,8 @@ RETCODE	Exec_with_parameters_resolved(StatementClass *stmt, BOOL *exec_end)
 
 		if (kres = res->next, kres)
 		{
+			if (kres->fields)
+				CI_Destructor(kres->fields);
 			kres->fields = res->fields;
 			res->fields = NULL;
 			kres->num_fields = res->num_fields;
@@ -374,11 +376,10 @@ RETCODE		SQL_API
 PGAPI_Execute(
 			  HSTMT hstmt)
 {
-	static char *func = "PGAPI_Execute";
+	CSTR func = "PGAPI_Execute";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	APDFields	*apdopts;
 	IPDFields	*ipdopts;
-	ConnectionClass *conn;
 	int			i,
 				retval, start_row, end_row;
 	BOOL	exec_end, recycled = FALSE, recycle = TRUE;
@@ -426,15 +427,6 @@ PGAPI_Execute(
 
 	SC_clear_error(stmt);
 
-	conn = SC_get_conn(stmt);
-	if (conn->status == CONN_EXECUTING)
-	{
-		SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Connection is already in use.");
-		SC_log_error(func, "", stmt);
-		mylog("%s: problem with connection\n", func);
-		return SQL_ERROR;
-	}
-
 	if (!stmt->statement)
 	{
 		SC_set_error(stmt, STMT_NO_STMTSTRING, "This handle does not have a SQL statement stored in it");
@@ -606,7 +598,7 @@ PGAPI_Transact(
 			   HDBC hdbc,
 			   UWORD fType)
 {
-	static char *func = "PGAPI_Transact";
+	CSTR func = "PGAPI_Transact";
 	extern ConnectionClass *conns[];
 	ConnectionClass *conn;
 	QResultClass *res;
@@ -684,7 +676,7 @@ RETCODE		SQL_API
 PGAPI_Cancel(
 			 HSTMT hstmt)		/* Statement to cancel. */
 {
-	static char *func = "PGAPI_Cancel";
+	CSTR func = "PGAPI_Cancel";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	ConnectionClass *conn;
 	RETCODE		result;
@@ -775,7 +767,7 @@ PGAPI_NativeSql(
 				SDWORD cbSqlStrMax,
 				SDWORD FAR * pcbSqlStr)
 {
-	static char *func = "PGAPI_NativeSql";
+	CSTR func = "PGAPI_NativeSql";
 	int			len = 0;
 	char	   *ptr;
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
@@ -824,7 +816,7 @@ PGAPI_ParamData(
 				HSTMT hstmt,
 				PTR FAR * prgbValue)
 {
-	static char *func = "PGAPI_ParamData";
+	CSTR func = "PGAPI_ParamData";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	APDFields	*apdopts;
 	IPDFields	*ipdopts;
@@ -885,7 +877,8 @@ PGAPI_ParamData(
 		retval = Exec_with_parameters_resolved(stmt, &exec_end);
 		if (exec_end)
 			return retval;
-		return PGAPI_Execute(stmt);
+		if (retval = PGAPI_Execute(stmt), SQL_NEED_DATA != retval)
+			return retval;
 	}
 
 	/*
@@ -921,7 +914,7 @@ PGAPI_PutData(
 			  PTR rgbValue,
 			  SDWORD cbValue)
 {
-	static char *func = "PGAPI_PutData";
+	CSTR func = "PGAPI_PutData";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	ConnectionClass *conn;
 	APDFields	*apdopts;
