@@ -16,7 +16,7 @@ RETCODE		SQL_API
 PGAPI_GetInfo30(HDBC hdbc, UWORD fInfoType, PTR rgbInfoValue,
 				SWORD cbInfoValueMax, SWORD FAR * pcbInfoValue)
 {
-	static char *func = "PGAPI_GetInfo30";
+	CSTR func = "PGAPI_GetInfo30";
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
 	ConnInfo	*ci = &(conn->connInfo);
 	char	   *p = NULL;
@@ -73,11 +73,10 @@ PGAPI_GetInfo30(HDBC hdbc, UWORD fInfoType, PTR rgbInfoValue,
 				| SQL_CA2_CRC_EXACT
 				| SQL_CA2_SENSITIVITY_DELETIONS
 				| SQL_CA2_SENSITIVITY_UPDATES
-				/* | SQL_CA2_SENSITIVITY_ADDITIONS */
+				| SQL_CA2_SENSITIVITY_ADDITIONS
 				);
 			if (ci->drivers.lie)
-				value |= (SQL_CA2_READ_ONLY_CONCURRENCY
-				| SQL_CA2_LOCK_CONCURRENCY
+				value |= (SQL_CA2_LOCK_CONCURRENCY
 				| SQL_CA2_OPT_VALUES_CONCURRENCY
 				| SQL_CA2_MAX_ROWS_SELECT
 				| SQL_CA2_MAX_ROWS_INSERT
@@ -100,6 +99,9 @@ PGAPI_GetInfo30(HDBC hdbc, UWORD fInfoType, PTR rgbInfoValue,
 			if (ci->updatable_cursors)
 				value |= (SQL_CA1_POS_UPDATE | SQL_CA1_POS_DELETE
 				| SQL_CA1_BULK_ADD
+				| SQL_CA1_BULK_UPDATE_BY_BOOKMARK
+				| SQL_CA1_BULK_DELETE_BY_BOOKMARK
+				| SQL_CA1_BULK_FETCH_BY_BOOKMARK
 				);
 			break;
 		case SQL_STATIC_CURSOR_ATTRIBUTES2:
@@ -108,9 +110,9 @@ PGAPI_GetInfo30(HDBC hdbc, UWORD fInfoType, PTR rgbInfoValue,
 			if (ci->updatable_cursors)
 				value |= (SQL_CA2_OPT_ROWVER_CONCURRENCY
 				| SQL_CA2_CRC_EXACT
-				/* | SQL_CA2_SENSITIVITY_ADDITIONS
+				| SQL_CA2_SENSITIVITY_ADDITIONS
 				| SQL_CA2_SENSITIVITY_DELETIONS
-				| SQL_CA2_SENSITIVITY_UPDATES */
+				| SQL_CA2_SENSITIVITY_UPDATES
 				);
 			break;
 
