@@ -35,6 +35,8 @@ SQLAllocHandle(SQLSMALLINT HandleType,
 			   SQLHANDLE InputHandle, SQLHANDLE * OutputHandle)
 {
 	RETCODE		ret;
+	ConnectionClass	*conn;
+
 	mylog("[[SQLAllocHandle]]");
 	switch (HandleType)
 	{
@@ -51,6 +53,13 @@ SQLAllocHandle(SQLSMALLINT HandleType,
 			ret = PGAPI_AllocStmt(InputHandle, OutputHandle);
 			LEAVE_CONN_CS((ConnectionClass *) InputHandle);
 			break;
+		case SQL_HANDLE_DESC:
+			conn = (ConnectionClass *) InputHandle;
+			ENTER_CONN_CS(conn);
+			CC_set_error(conn, CONN_NOT_IMPLEMENTED_ERROR, "can't alloc Desc Handle yet"); 
+			LEAVE_CONN_CS(conn);
+			ret = SQL_ERROR;
+			break;
 		default:
 			ret = SQL_ERROR;
 			break;
@@ -187,7 +196,7 @@ RETCODE		SQL_API
 SQLFetchScroll(HSTMT StatementHandle,
 			   SQLSMALLINT FetchOrientation, SQLINTEGER FetchOffset)
 {
-	static char *func = "SQLFetchScroll";
+	CSTR func = "SQLFetchScroll";
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	RETCODE		ret = SQL_SUCCESS;
 	IRDFields	*irdopts = SC_get_IRD(stmt);
@@ -213,8 +222,13 @@ mylog("bookmark=%u FetchOffset = %d\n", FetchOffset, bkmarkoff);
 		}
 	}
 	if (SQL_SUCCESS == ret)
+	{
+		ARDFields	*opts = SC_get_ARD(stmt);
+
 		ret = PGAPI_ExtendedFetch(StatementHandle, FetchOrientation, FetchOffset,
-				pcRow, rowStatusArray, bkmarkoff);
+				pcRow, rowStatusArray, bkmarkoff, opts->size_of_rowset);
+		stmt->transition_status = 6;
+	}
 	LEAVE_STMT_CS(stmt);
 	if (ret != SQL_SUCCESS)
 		mylog("%s return = %d\n", func, ret);
@@ -362,7 +376,7 @@ SQLGetStmtAttr(HSTMT StatementHandle,
 			   SQLINTEGER BufferLength, SQLINTEGER *StringLength)
 {
 	RETCODE	ret;
-	static char *func = "SQLGetStmtAttr";
+	CSTR func = "SQLGetStmtAttr";
 
 	mylog("[[%s]] Handle=%u %d\n", func, StatementHandle, Attribute);
 	ENTER_STMT_CS((StatementClass *) StatementHandle);
@@ -414,7 +428,7 @@ SQLSetDescRec(SQLHDESC DescriptorHandle,
 			  PTR Data, SQLINTEGER *StringLength,
 			  SQLINTEGER *Indicator)
 {
-	const char *func = "SQLSetDescRec";
+	CSTR func = "SQLSetDescRec";
 
 	mylog("[[SQLSetDescRec]]\n");
 	mylog("Error not implemented\n");
@@ -488,7 +502,7 @@ SQLSetStmtAttr(HSTMT StatementHandle,
 			   SQLINTEGER Attribute, PTR Value,
 			   SQLINTEGER StringLength)
 {
-	static char *func = "SQLSetStmtAttr";
+	CSTR func = "SQLSetStmtAttr";
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	RETCODE	ret;
 
@@ -564,7 +578,8 @@ PGAPI_GetFunctions30(HDBC hdbc, UWORD fFunction, UWORD FAR * pfExists)
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLSPECIALCOLUMNS);	/* 52 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLSTATISTICS);		/* 53 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLTABLES); /* 54 */
-	SQL_FUNC_ESET(pfExists, SQL_API_SQLBROWSECONNECT);	/* 55 */
+	if (ci->drivers.lie)
+		SQL_FUNC_ESET(pfExists, SQL_API_SQLBROWSECONNECT); /* 55 not implemented yet */
 	if (ci->drivers.lie)
 		SQL_FUNC_ESET(pfExists, SQL_API_SQLCOLUMNPRIVILEGES); /* 56 not implemented yet */ 
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLDATASOURCES);	/* 57 */
@@ -626,7 +641,7 @@ RETCODE	SQL_API
 SQLBulkOperations(HSTMT hstmt, SQLSMALLINT operation)
 {
 	RETCODE	ret;
-	static char	*func = "SQLBulkOperations";
+	CSTR func = "SQLBulkOperations";
 	StatementClass	*stmt = (StatementClass *) hstmt;
 
 	ENTER_STMT_CS(stmt);
