@@ -36,7 +36,7 @@ set_statement_option(ConnectionClass *conn,
 					 UWORD fOption,
 					 UDWORD vParam)
 {
-	static char *func = "set_statement_option";
+	CSTR func = "set_statement_option";
 	char		changed = FALSE;
 	ConnInfo   *ci = NULL;
 	UDWORD		setval;
@@ -68,7 +68,7 @@ set_statement_option(ConnectionClass *conn,
 			setval = SQL_CONCUR_READ_ONLY;
 			if (SQL_CONCUR_READ_ONLY == vParam)
 				;
-			if (ci->drivers.lie)
+			else if (ci->drivers.lie)
 				setval = vParam;
 			else if (ci->updatable_cursors)
 				setval = SQL_CONCUR_ROWVER;
@@ -215,7 +215,7 @@ set_statement_option(ConnectionClass *conn,
 			 */
 
 			if (stmt && stmt->save_rowset_size <= 0 && stmt->last_fetch_count > 0)
-				stmt->save_rowset_size = SC_get_ARD(stmt)->rowset_size;
+				stmt->save_rowset_size = SC_get_ARD(stmt)->size_of_rowset_odbc2;
 
 			if (vParam < 1)
 			{
@@ -224,9 +224,9 @@ set_statement_option(ConnectionClass *conn,
 			}
 
 			if (conn)
-				conn->ardOptions.rowset_size = vParam;
+				conn->ardOptions.size_of_rowset_odbc2 = vParam;
 			if (stmt)
-				SC_get_ARD(stmt)->rowset_size = vParam;
+				SC_get_ARD(stmt)->size_of_rowset_odbc2 = vParam;
 			break;
 
 		case SQL_SIMULATE_CURSOR:		/* NOT SUPPORTED */
@@ -305,7 +305,7 @@ PGAPI_SetConnectOption(
 					   UWORD fOption,
 					   UDWORD vParam)
 {
-	static char *func = "PGAPI_SetConnectOption";
+	CSTR func = "PGAPI_SetConnectOption";
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
 	char		changed = FALSE;
 	RETCODE		retval;
@@ -480,7 +480,7 @@ PGAPI_SetConnectOption(
 					char *asPara;
 					if (conn->unicode)
 					{
-						asPara = ucs2_to_utf8((SQLWCHAR *) vParam, -1, NULL);
+						asPara = ucs2_to_utf8((SQLWCHAR *) vParam, -1, NULL, FALSE);
 						cmp = strcmp(asPara, "Microsoft Jet");
 						free(asPara);
 					}
@@ -517,7 +517,7 @@ PGAPI_GetConnectOption(
 					   UWORD fOption,
 					   PTR pvParam)
 {
-	static char *func = "PGAPI_GetConnectOption";
+	CSTR func = "PGAPI_GetConnectOption";
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
 	ConnInfo   *ci = &(conn->connInfo);
 
@@ -603,7 +603,7 @@ PGAPI_SetStmtOption(
 					UWORD fOption,
 					UDWORD vParam)
 {
-	static char *func = "PGAPI_SetStmtOption";
+	CSTR func = "PGAPI_SetStmtOption";
 	StatementClass *stmt = (StatementClass *) hstmt;
 
 	mylog("%s: entering...\n", func);
@@ -629,10 +629,11 @@ PGAPI_GetStmtOption(
 					UWORD fOption,
 					PTR pvParam)
 {
-	static char *func = "PGAPI_GetStmtOption";
+	CSTR func = "PGAPI_GetStmtOption";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	QResultClass *res;
 	ConnInfo   *ci = &(SC_get_conn(stmt)->connInfo);
+	int	ridx;
 
 	mylog("%s: entering...\n", func);
 
@@ -653,12 +654,19 @@ PGAPI_GetStmtOption(
 		case SQL_ROW_NUMBER:
 
 			res = SC_get_Curres(stmt);
+			if (!res)
+			{
+				SC_set_error(stmt, STMT_INVALID_CURSOR_STATE_ERROR, "The cursor has no result.");
+				SC_log_error(func, "", stmt);
+				return SQL_ERROR;
+			}
 
-			if (stmt->manual_result || !ci->drivers.use_declarefetch)
+			ridx = GIdx2ResultIdx(stmt->currTuple, stmt, res);
+			if (stmt->manual_result || !SC_is_fetchcursor(stmt))
 			{
 				/* make sure we're positioned on a valid row */
-				if ((stmt->currTuple < 0) ||
-					(stmt->currTuple >= QR_get_num_backend_tuples(res)))
+				if ((ridx < 0) ||
+					(ridx >= QR_get_num_backend_tuples(res)))
 				{
 					SC_set_error(stmt, STMT_INVALID_CURSOR_STATE_ERROR, "Not positioned on a valid row.");
 					SC_log_error(func, "", stmt);
@@ -667,7 +675,7 @@ PGAPI_GetStmtOption(
 			}
 			else
 			{
-				if (stmt->currTuple == -1 || !res || !res->tupleField)
+				if (stmt->currTuple < 0 || !res->tupleField)
 				{
 					SC_set_error(stmt, STMT_INVALID_CURSOR_STATE_ERROR, "Not positioned on a valid row.");
 					SC_log_error(func, "", stmt);
@@ -731,7 +739,7 @@ PGAPI_GetStmtOption(
 			break;
 
 		case SQL_ROWSET_SIZE:
-			*((SDWORD *) pvParam) = SC_get_ARD(stmt)->rowset_size;
+			*((SDWORD *) pvParam) = SC_get_ARD(stmt)->size_of_rowset_odbc2;
 			break;
 
 		case SQL_SIMULATE_CURSOR:		/* NOT SUPPORTED */
