@@ -309,9 +309,7 @@ void lower_the_name(char *name, ConnectionClass *conn, BOOL dquote)
 		for (ptr = name; *ptr; ptr++)
 		{
 			encoded_nextchar(&encstr);
-			if (ENCODE_STATUS(encstr) != 0)
-				ptr++;
-			else
+			if (ENCODE_STATUS(encstr) == 0)
 				*ptr = tolower((unsigned char) *ptr);
 		}
 	}
@@ -320,7 +318,7 @@ void lower_the_name(char *name, ConnectionClass *conn, BOOL dquote)
 char
 parse_statement(StatementClass *stmt)
 {
-	static char *func = "parse_statement";
+	CSTR		func = "parse_statement";
 	char		token[256], stoken[256];
 	char		delim,
 				quote,
@@ -927,7 +925,7 @@ parse_statement(StatementClass *stmt)
 			 	 	 * We also have to check as follows.
 			 	 	 */
 					sprintf(token, "select nspname from pg_namespace n, pg_class c"
-						" where c.relnamespace=n.oid and c.oid='%s'::regclass", ti[i]->name);
+						" where c.relnamespace=n.oid and c.oid='\"%s\"'::regclass", ti[i]->name);
 					res = CC_send_query(conn, token, NULL, CLEAR_RESULT_ON_ABORT);
 					if (res)
 					{
