@@ -36,7 +36,7 @@ PGAPI_GetDiagRec(SQLSMALLINT HandleType, SQLHANDLE Handle,
 		SQLSMALLINT BufferLength, SQLSMALLINT *TextLength)
 {
 	RETCODE		ret;
-	static const char *func = "PGAPI_GetDiagRec";
+	CSTR func = "PGAPI_GetDiagRec";
 
 	mylog("%s entering rec=%d", func, RecNumber);
 	switch (HandleType)
@@ -85,7 +85,7 @@ PGAPI_GetDiagField(SQLSMALLINT HandleType, SQLHANDLE Handle,
 	StatementClass	*stmt;
 	SDWORD		rc;
 	SWORD		pcbErrm;
-	static const char *func = "PGAPI_GetDiagField";
+	CSTR func = "PGAPI_GetDiagField";
 
 	mylog("%s entering rec=%d", func, RecNumber);
 	switch (HandleType)
@@ -295,6 +295,18 @@ PGAPI_GetDiagField(SQLSMALLINT HandleType, SQLHANDLE Handle,
 						*StringLengthPtr = sizeof(SQLINTEGER);
 					ret = SQL_SUCCESS;
 					break;
+				case SQL_DIAG_ROW_NUMBER:
+					*((SQLINTEGER *) DiagInfoPtr) = SQL_ROW_NUMBER_UNKNOWN;
+					if (StringLengthPtr)
+						*StringLengthPtr = sizeof(SQLINTEGER);
+					ret = SQL_SUCCESS;
+					break;
+				case SQL_DIAG_COLUMN_NUMBER:
+					*((SQLINTEGER *) DiagInfoPtr) = SQL_COLUMN_NUMBER_UNKNOWN;
+					if (StringLengthPtr)
+						*StringLengthPtr = sizeof(SQLINTEGER);
+					ret = SQL_SUCCESS;
+					break;
 				case SQL_DIAG_RETURNCODE: /* driver manager returns */
 					break;
 			}
@@ -350,7 +362,7 @@ PGAPI_GetConnectAttr(HDBC ConnectionHandle,
 			SQLINTEGER Attribute, PTR Value,
 			SQLINTEGER BufferLength, SQLINTEGER *StringLength)
 {
-	static const char *func = "PGAPI_GetConnectAttr";
+	CSTR func = "PGAPI_GetConnectAttr";
 	ConnectionClass *conn = (ConnectionClass *) ConnectionHandle;
 	RETCODE	ret = SQL_SUCCESS;
 	SQLINTEGER	len = 4;
@@ -371,9 +383,8 @@ PGAPI_GetConnectAttr(HDBC ConnectionHandle,
 			*((SQLUINTEGER *) Value) = 0;
 			break;
 		case SQL_ATTR_METADATA_ID:
-			CC_set_error(conn, STMT_INVALID_OPTION_IDENTIFIER, "Unsupported connect attribute (Get)");
-			CC_log_error(func, "", conn);
-			return SQL_ERROR;
+			*((SQLUINTEGER *) Value) = conn->stmtOptions.metadata_id;
+			break;
 		default:
 			ret = PGAPI_GetConnectOption(ConnectionHandle, (UWORD) Attribute, Value);
 	}
@@ -466,7 +477,7 @@ ARDSetField(StatementClass *stmt, SQLSMALLINT RecNumber,
 	switch (FieldIdentifier)
 	{
 		case SQL_DESC_ARRAY_SIZE:
-			opts->rowset_size = (SQLUINTEGER) Value;
+			opts->size_of_rowset = (SQLUINTEGER) Value;
 			return ret; 
 		case SQL_DESC_ARRAY_STATUS_PTR:
 			opts->row_operation_ptr = Value;
@@ -566,7 +577,7 @@ ARDSetField(StatementClass *stmt, SQLSMALLINT RecNumber,
 			opts->bindings[row_idx].buflen = (Int4) Value;
 			break;
 		case SQL_DESC_PRECISION:
-			opts->bindings[row_idx].precision = (Int2) Value;
+			opts->bindings[row_idx].precision = (Int2)((Int4) Value);
 			break;
 		case SQL_DESC_SCALE:
 			opts->bindings[row_idx].scale = (Int4) Value;
@@ -713,10 +724,10 @@ APDSetField(StatementClass *stmt, SQLSMALLINT RecNumber,
 			opts->parameters[para_idx].used = Value;
 			break;
 		case SQL_DESC_PRECISION:
-			opts->parameters[para_idx].precision = (Int2) Value;
+			opts->parameters[para_idx].precision = (Int2) ((Int4) Value);
 			break;
 		case SQL_DESC_SCALE:
-			opts->parameters[para_idx].scale = (Int2) Value;
+			opts->parameters[para_idx].scale = (Int2) ((Int4) Value);
 			break;
 		case SQL_DESC_ALLOC_TYPE: /* read-only */
 		case SQL_DESC_DATETIME_INTERVAL_PRECISION:
@@ -853,10 +864,10 @@ IPDSetField(StatementClass *stmt, SQLSMALLINT RecNumber,
 			ipdopts->parameters[para_idx].SQLType = (Int4) Value;
 			break;
 		case SQL_DESC_PARAMETER_TYPE:
-			ipdopts->parameters[para_idx].paramType = (Int2) Value;
+			ipdopts->parameters[para_idx].paramType = (Int2) ((Int4) Value);
 			break;
 		case SQL_DESC_SCALE:
-			ipdopts->parameters[para_idx].decimal_digits = (Int2) Value;
+			ipdopts->parameters[para_idx].decimal_digits = (Int2) ((Int4) Value);
 			break;
 		case SQL_DESC_ALLOC_TYPE: /* read-only */ 
 		case SQL_DESC_CASE_SENSITIVE: /* read-only */
@@ -937,7 +948,7 @@ ARDGetField(StatementClass *stmt, SQLSMALLINT RecNumber,
 	switch (FieldIdentifier)
 	{
 		case SQL_DESC_ARRAY_SIZE:
-			ival = opts->rowset_size;
+			ival = opts->size_of_rowset;
 			break; 
 		case SQL_DESC_ARRAY_STATUS_PTR:
 			rettype = SQL_IS_POINTER;
@@ -1397,7 +1408,7 @@ PGAPI_GetStmtAttr(HSTMT StatementHandle,
 		SQLINTEGER Attribute, PTR Value,
 		SQLINTEGER BufferLength, SQLINTEGER *StringLength)
 {
-	static char *func = "PGAPI_GetStmtAttr";
+	CSTR func = "PGAPI_GetStmtAttr";
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	RETCODE		ret = SQL_SUCCESS;
 	int			len = 0;
@@ -1450,7 +1461,7 @@ PGAPI_GetStmtAttr(HSTMT StatementHandle,
 			len = 4;
 			break;
 		case SQL_ATTR_ROW_ARRAY_SIZE:	/* 27 */
-			*((SQLUINTEGER *) Value) = SC_get_ARD(stmt)->rowset_size;
+			*((SQLUINTEGER *) Value) = SC_get_ARD(stmt)->size_of_rowset;
 			len = 4;
 			break;
 		case SQL_ATTR_APP_ROW_DESC:		/* 10010 */
@@ -1460,18 +1471,27 @@ PGAPI_GetStmtAttr(HSTMT StatementHandle,
 			len = 4;
 			*((HSTMT *) Value) = descHandleFromStatementHandle(StatementHandle, Attribute); 
 			break;
-		case SQL_ATTR_AUTO_IPD:	/* 10001 */
-			/* case SQL_ATTR_ROW_BIND_TYPE: ** == SQL_BIND_TYPE(ODBC2.0) */
 
 		case SQL_ATTR_CURSOR_SCROLLABLE:		/* -1 */
+			len = 4;
+			if (SQL_CURSOR_FORWARD_ONLY == stmt->options.cursor_type)
+				*((SQLUINTEGER *) Value) = SQL_NONSCROLLABLE;
+			else
+				*((SQLUINTEGER *) Value) = SQL_SCROLLABLE;
+			break;
 		case SQL_ATTR_CURSOR_SENSITIVITY:		/* -2 */
-		case SQL_ATTR_ENABLE_AUTO_IPD:	/* 15 */
+			len = 4;
+			if (SQL_CONCUR_READ_ONLY == stmt->options.scroll_concurrency)
+				*((SQLUINTEGER *) Value) = SQL_INSENSITIVE;
+			else
+				*((SQLUINTEGER *) Value) = SQL_UNSPECIFIED;
+			break;
 		case SQL_ATTR_METADATA_ID:		/* 10014 */
-
-			/*
-			 * case SQL_ATTR_PREDICATE_PTR: case
-			 * SQL_ATTR_PREDICATE_OCTET_LENGTH_PTR:
-			 */
+			*((SQLUINTEGER *) Value) = stmt->options.metadata_id;
+			break;
+		case SQL_ATTR_AUTO_IPD:	/* 10001 */
+			/* case SQL_ATTR_ROW_BIND_TYPE: ** == SQL_BIND_TYPE(ODBC2.0) */
+		case SQL_ATTR_ENABLE_AUTO_IPD:	/* 15 */
 			SC_set_error(stmt, STMT_INVALID_OPTION_IDENTIFIER, "Unsupported statement option (Get)");
 			SC_log_error(func, "", stmt);
 			return SQL_ERROR;
@@ -1496,11 +1516,13 @@ PGAPI_SetConnectAttr(HDBC ConnectionHandle,
 	mylog("PGAPI_SetConnectAttr %d\n", Attribute);
 	switch (Attribute)
 	{
+		case SQL_ATTR_METADATA_ID:
+			conn->stmtOptions.metadata_id = (SQLUINTEGER) Value;
+			break;
 		case SQL_ATTR_ASYNC_ENABLE:
 		case SQL_ATTR_AUTO_IPD:
 		case SQL_ATTR_CONNECTION_DEAD:
 		case SQL_ATTR_CONNECTION_TIMEOUT:
-		case SQL_ATTR_METADATA_ID:
 		case SQL_ATTR_ENLIST_IN_DTC:
 			CC_set_error(conn, STMT_OPTION_NOT_FOR_THE_DRIVER, "Unsupported connect attribute (Set)");
 			return SQL_ERROR;
@@ -1521,7 +1543,7 @@ PGAPI_GetDescField(SQLHDESC DescriptorHandle,
 	HSTMT		hstmt;
 	SQLUINTEGER	descType;
 	StatementClass *stmt;
-	static const char *func = "PGAPI_GetDescField";
+	CSTR func = "PGAPI_GetDescField";
 
 	mylog("%s h=%u rec=%d field=%d blen=%d\n", func, DescriptorHandle, RecNumber, FieldIdentifier, BufferLength);
 	hstmt = statementHandleFromDescHandle(DescriptorHandle, &descType);
@@ -1576,7 +1598,7 @@ PGAPI_SetDescField(SQLHDESC DescriptorHandle,
 	HSTMT		hstmt;
 	SQLUINTEGER	descType;
 	StatementClass *stmt;
-	static const char *func = "PGAPI_SetDescField";
+	CSTR func = "PGAPI_SetDescField";
 
 	mylog("%s h=%u rec=%d field=%d val=%x,%d\n", func, DescriptorHandle, RecNumber, FieldIdentifier, Value, BufferLength);
 	hstmt = statementHandleFromDescHandle(DescriptorHandle, &descType);
@@ -1627,7 +1649,7 @@ PGAPI_SetStmtAttr(HSTMT StatementHandle,
 		SQLINTEGER Attribute, PTR Value,
 		SQLINTEGER StringLength)
 {
-	static char *func = "PGAPI_SetStmtAttr";
+	CSTR func = "PGAPI_SetStmtAttr";
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 
 	mylog("%s Handle=%u %d,%u\n", func, StatementHandle, Attribute, Value);
@@ -1644,7 +1666,6 @@ PGAPI_SetStmtAttr(HSTMT StatementHandle,
 		/* case SQL_ATTR_ROW_BIND_TYPE: ** == SQL_BIND_TYPE(ODBC2.0) */
 		case SQL_ATTR_IMP_ROW_DESC:	/* 10012 (read-only) */
 		case SQL_ATTR_IMP_PARAM_DESC:	/* 10013 (read-only) */
-		case SQL_ATTR_METADATA_ID:		/* 10014 */
 
 			/*
 			 * case SQL_ATTR_PREDICATE_PTR: case
@@ -1654,6 +1675,9 @@ PGAPI_SetStmtAttr(HSTMT StatementHandle,
 			SC_log_error(func, "", stmt);
 			return SQL_ERROR;
 
+		case SQL_ATTR_METADATA_ID:		/* 10014 */
+			stmt->options.metadata_id = (SQLUINTEGER) Value;
+			break;
 		case SQL_ATTR_FETCH_BOOKMARK_PTR:		/* 16 */
 			stmt->options.bookmark_ptr = Value;
 			break;
@@ -1688,7 +1712,7 @@ PGAPI_SetStmtAttr(HSTMT StatementHandle,
 			SC_get_IRD(stmt)->rowsFetched = (SQLUINTEGER *) Value;
 			break;
 		case SQL_ATTR_ROW_ARRAY_SIZE:	/* 27 */
-			SC_get_ARD(stmt)->rowset_size = (SQLUINTEGER) Value;
+			SC_get_ARD(stmt)->size_of_rowset = (SQLUINTEGER) Value;
 			break;
 		default:
 			return PGAPI_SetStmtOption(StatementHandle, (UWORD) Attribute, (UDWORD) Value);
@@ -1697,18 +1721,24 @@ PGAPI_SetStmtAttr(HSTMT StatementHandle,
 }
 
 #ifdef DRIVER_CURSOR_IMPLEMENT
+#define	CALC_BOOKMARK_ADDR(book, offset, bind_size, index) \
+	(book->buffer + offset + \
+	(bind_size > 0 ? bind_size : (SQL_C_VARBOOKMARK == book->returntype ? book->buflen : sizeof(UInt4))) * index)    	
+
 RETCODE	SQL_API
 PGAPI_BulkOperations(HSTMT hstmt, SQLSMALLINT operation)
 {
-	static char	*func = "PGAPI_BulkOperations";
+	CSTR func = "PGAPI_BulkOperations";
 	StatementClass	*stmt = (StatementClass *) hstmt;
 	ARDFields	*opts = SC_get_ARD(stmt);
 	RETCODE		ret;
-	UInt4		offset, bind_size = opts->bind_size, *bmark = NULL;
+	UInt4		offset, bind_size = opts->bind_size,
+			global_idx;
 	int		i, processed;
 	ConnectionClass	*conn;
 	BOOL		auto_commit_needed = FALSE;
 	QResultClass	*res;
+	BindInfoClass	*bookmark;
 
 	mylog("%s operation = %d\n", func, operation);
 	SC_clear_error(stmt);
@@ -1722,15 +1752,19 @@ PGAPI_BulkOperations(HSTMT hstmt, SQLSMALLINT operation)
 	}
 	if (SQL_ADD != operation)
 	{
-		if (bmark = (UInt4 *) opts->bookmark->buffer, !bmark)
+		if (!(bookmark = opts->bookmark) || !(bookmark->buffer))
 		{
 			SC_set_error(stmt, STMT_INVALID_OPTION_IDENTIFIER, "bookmark isn't specified");
 			return SQL_ERROR;
 		}
-		bmark += (offset >> 4);
 	}
-	for (i = 0, processed = 0; i < opts->rowset_size; i++)
+	for (i = 0, processed = 0; i < opts->size_of_rowset; i++)
 	{
+		if (SQL_ADD != operation)
+		{
+			memcpy(&global_idx, CALC_BOOKMARK_ADDR(bookmark, offset, bind_size, i), sizeof(UInt4));
+			global_idx--;
+		}
 		/* Note opts->row_operation_ptr is ignored */
 		switch (operation)
 		{
@@ -1738,25 +1772,18 @@ PGAPI_BulkOperations(HSTMT hstmt, SQLSMALLINT operation)
 				ret = SC_pos_add(stmt, (UWORD) i);
 				break;
 			case SQL_UPDATE_BY_BOOKMARK:
-				ret = SC_pos_update(stmt, (UWORD) i, *bmark);
+				ret = SC_pos_update(stmt, (UWORD) i, global_idx);
 				break;
 			case SQL_DELETE_BY_BOOKMARK:
-				ret = SC_pos_delete(stmt, (UWORD) i, *bmark);
+				ret = SC_pos_delete(stmt, (UWORD) i, global_idx);
 				break;
 			case SQL_FETCH_BY_BOOKMARK:
-				ret = SC_pos_refresh(stmt, (UWORD) i, *bmark);
+				ret = SC_pos_refresh(stmt, (UWORD) i, global_idx);
 				break;
 		}
 		processed++;
 		if (SQL_ERROR == ret)
 			break;
-		if (SQL_ADD != operation)
-		{
-			if (bind_size > 0)
-				bmark += (bind_size >> 2);
-			else
-				bmark++;
-		} 
 	}
 	if (auto_commit_needed)
 		PGAPI_SetConnectOption(conn, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_ON);
