@@ -41,6 +41,7 @@
 void
 QR_set_num_fields(QResultClass *self, int new_num_fields)
 {
+	if (!self)	return;
 	mylog("in QR_set_num_fields\n");
 
 	CI_set_num_fields(self->fields, new_num_fields);
@@ -143,11 +144,15 @@ QR_Constructor()
 void
 QR_Destructor(QResultClass *self)
 {
+	if (!self)	return;
 	mylog("QResult: in DESTRUCTOR\n");
 
 	/* manual result set tuples */
 	if (self->manual_tuples)
+	{
 		TL_Destructor(self->manual_tuples);
+		self->manual_tuples = NULL;
+	}
 
 	/*
 	 * If conn is defined, then we may have used "backend_tuples", so in
@@ -160,26 +165,44 @@ QR_Destructor(QResultClass *self)
 
 	/* Should have been freed in the close() but just in case... */
 	if (self->cursor)
+	{
 		free(self->cursor);
+		self->cursor = NULL;
+	}
 
 	/* Free up column info */
 	if (self->fields)
+	{
 		CI_Destructor(self->fields);
+		self->fields = NULL;
+	}
 
 	/* Free command info (this is from strdup()) */
 	if (self->command)
+	{
 		free(self->command);
+		self->command = NULL;
+	}
 
 	/* Free message info (this is from strdup()) */
 	if (self->message)
+	{
 		free(self->message);
+		self->message = NULL;
+	}
 
 	/* Free notice info (this is from strdup()) */
 	if (self->notice)
+	{
 		free(self->notice);
+		self->notice = NULL;
+	}
 	/* Destruct the result object in the chain */
 	if (self->next)
+	{
 		QR_Destructor(self->next);
+		self->next = NULL;
+	}
 
 	free(self);
 
@@ -708,11 +731,10 @@ inolog("clear obsolete %d tuples\n", num_backend_rows);
 				mylog("end of tuple list -- setting inUse to false: this = %u\n", self);
 
 				self->inTuples = FALSE;
-				if (self->num_total_rows > 0)
+				qlog("    [ fetched %d rows ]\n", self->num_total_rows);
+				mylog("_next_tuple: 'C' fetch_total = %d & this_fetch = %d\n", self->num_total_rows, self->num_backend_rows);
+				if (self->num_backend_rows > 0)
 				{
-					qlog("    [ fetched %d rows ]\n", self->num_total_rows);
-					mylog("_next_tuple: 'C' fetch_max && fcount = %d\n", self->num_total_rows);
-
 					/* set to first row */
 					self->tupleField = self->backend_tuples + (offset * self->num_fields);
 					return TRUE;
@@ -720,7 +742,6 @@ inolog("clear obsolete %d tuples\n", num_backend_rows);
 				else
 				{
 					/* We are surely done here (we read 0 tuples) */
-					qlog("    [ fetched 0 rows ]\n");
 					mylog("_next_tuple: 'C': DONE (fcount == 0)\n");
 					return -1;	/* end of tuples */
 				}
