@@ -38,7 +38,7 @@ PGAPI_RowCount(
 			   HSTMT hstmt,
 			   SDWORD FAR * pcrow)
 {
-	static char *func = "PGAPI_RowCount";
+	CSTR func = "PGAPI_RowCount";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	QResultClass *res;
 	ConnInfo   *ci;
@@ -98,7 +98,7 @@ PGAPI_NumResultCols(
 					HSTMT hstmt,
 					SWORD FAR * pccol)
 {
-	static char *func = "PGAPI_NumResultCols";
+	CSTR func = "PGAPI_NumResultCols";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	QResultClass *result;
 	char		parse_ok;
@@ -168,7 +168,7 @@ PGAPI_DescribeCol(
 				  SWORD FAR * pibScale,
 				  SWORD FAR * pfNullable)
 {
-	static char *func = "PGAPI_DescribeCol";
+	CSTR func = "PGAPI_DescribeCol";
 
 	/* gets all the information about a specific column */
 	StatementClass *stmt = (StatementClass *) hstmt;
@@ -276,7 +276,13 @@ PGAPI_DescribeCol(
 		if ((NULL == res) || ((stmt->status != STMT_FINISHED) && (stmt->status != STMT_PREMATURE)))
 		{
 			/* no query has been executed on this statement */
-			SC_set_error(stmt, STMT_SEQUENCE_ERROR, "No query has been assigned to this statement.");
+			SC_set_error(stmt, STMT_EXEC_ERROR, "No query has been assigned to this statement.");
+			SC_log_error(func, "", stmt);
+			return SQL_ERROR;
+		}
+		else if (!QR_command_maybe_successful(res))
+		{
+			SC_set_errornumber(stmt, STMT_EXEC_ERROR);
 			SC_log_error(func, "", stmt);
 			return SQL_ERROR;
 		}
@@ -382,7 +388,7 @@ PGAPI_ColAttributes(
 					SWORD FAR * pcbDesc,
 					SDWORD FAR * pfDesc)
 {
-	static char *func = "PGAPI_ColAttributes";
+	CSTR func = "PGAPI_ColAttributes";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	IRDFields	*irdflds;
 	Int4		col_idx, field_type = 0;
@@ -780,7 +786,7 @@ PGAPI_GetData(
 			  SDWORD cbValueMax,
 			  SDWORD FAR * pcbValue)
 {
-	static char *func = "PGAPI_GetData";
+	CSTR func = "PGAPI_GetData";
 	QResultClass *res;
 	StatementClass *stmt = (StatementClass *) hstmt;
 	int			num_cols,
@@ -833,7 +839,7 @@ PGAPI_GetData(
 #endif /* ODBCVER */
 				break;
 			default:
-inolog("Column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
+inolog("GetData Column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 				SC_set_error(stmt, STMT_PROGRAM_TYPE_OUT_OF_RANGE, "Column 0 is not of type SQL_C_BOOKMARK");
 				SC_log_error(func, "", stmt);
 				return SQL_ERROR;
@@ -875,9 +881,7 @@ inolog("Column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 				value = QR_get_value_manual(res, stmt->currTuple, icol);
 			else
 			{
-				Int4	curt = res->base;
-				if (stmt->rowset_start >= 0)
-					curt += (stmt->currTuple - stmt->rowset_start);
+				Int4	curt = GIdx2ResultIdx(stmt->currTuple, stmt, res);
 				value = QR_get_value_backend_row(res, curt, icol);
 			}
 			mylog("     value = '%s'\n", value);
@@ -901,12 +905,26 @@ inolog("Column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 
 	if (get_bookmark)
 	{
-		*((UDWORD *) rgbValue) = SC_get_bookmark(stmt);
+		BOOL	contents_get = FALSE;
 
+		if (rgbValue)
+		{
+			if (SQL_C_BOOKMARK == fCType || 4 <= cbValueMax)
+			{
+				contents_get = TRUE; 
+				*((UDWORD *) rgbValue) = SC_get_bookmark(stmt);
+			}
+		}
 		if (pcbValue)
-			*pcbValue = 4;
+			*pcbValue = sizeof(UDWORD);
 
-		return SQL_SUCCESS;
+		if (contents_get)
+			return SQL_SUCCESS;
+		else
+		{
+			SC_set_error(stmt, STMT_TRUNCATED, "The buffer was too small for the GetData.");
+			return SQL_SUCCESS_WITH_INFO;
+		}
 	}
 
 	field_type = QR_get_field_type(res, icol);
@@ -963,7 +981,7 @@ RETCODE		SQL_API
 PGAPI_Fetch(
 			HSTMT hstmt)
 {
-	static char *func = "PGAPI_Fetch";
+	CSTR func = "PGAPI_Fetch";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	ARDFields	*opts;
 	QResultClass *res;
@@ -1115,9 +1133,10 @@ PGAPI_ExtendedFetch(
 					SDWORD irow,
 					UDWORD FAR * pcrow,
 					UWORD FAR * rgfRowStatus,
-					SQLINTEGER bookmark_offset)
+					SQLINTEGER bookmark_offset,
+					SQLINTEGER rowsetSize)
 {
-	static char *func = "PGAPI_ExtendedFetch";
+	CSTR func = "PGAPI_ExtendedFetch";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	ARDFields	*opts;
 	QResultClass *res;
@@ -1202,7 +1221,7 @@ PGAPI_ExtendedFetch(
 
 	/* Initialize to no rows fetched */
 	if (rgfRowStatus)
-		for (i = 0; i < opts->rowset_size; i++)
+		for (i = 0; i < rowsetSize; i++)
 			*(rgfRowStatus + i) = SQL_ROW_NOROW;
 
 	if (pcrow)
@@ -1225,7 +1244,7 @@ PGAPI_ExtendedFetch(
 			 * SQL_FETCH_FIRST.
 			 */
 
-			progress_size = (save_rowset_size > 0 ? save_rowset_size : opts->rowset_size);
+			progress_size = (save_rowset_size > 0 ? save_rowset_size : rowsetSize);
 			if (stmt->rowset_start < 0)
 				stmt->rowset_start = 0;
 
@@ -1266,17 +1285,17 @@ PGAPI_ExtendedFetch(
 			}
 			if (stmt->rowset_start >= num_tuples)
 			{
-				if (opts->rowset_size > num_tuples)
+				if (rowsetSize > num_tuples)
 				{
 					SC_set_error(stmt, STMT_POS_BEFORE_RECORDSET, "fetch prior from eof and before the beggining");
 				}
-				stmt->rowset_start = num_tuples <= 0 ? 0 : (num_tuples - opts->rowset_size);
+				stmt->rowset_start = num_tuples <= 0 ? 0 : (num_tuples - rowsetSize);
 
 			}
 			else
 			{
 #ifdef	DRIVER_CURSOR_IMPLEMENT
-				if (i = getNthValid(res, stmt->rowset_start - 1, SQL_FETCH_PRIOR, opts->rowset_size, &stmt->rowset_start), i < -1)
+				if (i = getNthValid(res, stmt->rowset_start - 1, SQL_FETCH_PRIOR, rowsetSize, &stmt->rowset_start), i < -1)
 				{
 					SC_set_error(stmt, STMT_POS_BEFORE_RECORDSET, "fetch prior and before the beggining");
 					stmt->rowset_start = 0;
@@ -1306,7 +1325,7 @@ PGAPI_ExtendedFetch(
 		case SQL_FETCH_LAST:
 			mylog("SQL_FETCH_LAST: num_tuples=%d, currtuple=%d\n", num_tuples, stmt->currTuple);
 
-			stmt->rowset_start = num_tuples <= 0 ? 0 : (num_tuples - opts->rowset_size);
+			stmt->rowset_start = num_tuples <= 0 ? 0 : (num_tuples - rowsetSize);
 			break;
 
 		case SQL_FETCH_ABSOLUTE:
@@ -1422,7 +1441,7 @@ PGAPI_ExtendedFetch(
 	/* If *new* rowset is prior to result_set, return no data found */
 	if (stmt->rowset_start < 0)
 	{
-		if (stmt->rowset_start + opts->rowset_size <= 0)
+		if (stmt->rowset_start + rowsetSize <= 0)
 		{
 			EXTFETCH_RETURN_BOF(stmt, res)
 		}
@@ -1434,10 +1453,10 @@ PGAPI_ExtendedFetch(
 	}
 
 	/* currTuple is always 1 row prior to the rowset */
-	stmt->currTuple = stmt->rowset_start - 1;
+	stmt->currTuple = RowIdx2GIdx(-1, stmt);
 
 	/* increment the base row in the tuple cache */
-	QR_set_rowset_size(res, opts->rowset_size);
+	QR_set_rowset_size(res, rowsetSize);
 	if (SC_is_fetchcursor(stmt))
 		QR_inc_base(res, stmt->last_fetch_count_include_ommitted);
 	else
@@ -1452,7 +1471,7 @@ PGAPI_ExtendedFetch(
 	mylog("PGAPI_ExtendedFetch: new currTuple = %d\n", stmt->currTuple);
 
 	truncated = error = FALSE;
-	for (i = 0, currp = stmt->rowset_start; i < opts->rowset_size; currp++)
+	for (i = 0, currp = stmt->rowset_start; i < rowsetSize; currp++)
 	{
 		stmt->bind_row = i;		/* set the binding location */
 		result = SC_fetch(stmt);
@@ -1504,13 +1523,13 @@ PGAPI_ExtendedFetch(
 
 	/* Save the fetch count for SQLSetPos */
 	stmt->last_fetch_count = i;
-	stmt->last_fetch_count_include_ommitted = currp - stmt->rowset_start;
+	stmt->last_fetch_count_include_ommitted = GIdx2RowIdx(currp, stmt);
 
 	/* Reset next binding row */
 	stmt->bind_row = 0;
 
 	/* Move the cursor position to the first row in the result set. */
-	stmt->currTuple = stmt->rowset_start;
+	stmt->currTuple = RowIdx2GIdx(0, stmt);
 
 	/* For declare/fetch, need to reset cursor to beginning of rowset */
 	if (SC_is_fetchcursor(stmt) && !stmt->manual_result)
@@ -1543,7 +1562,7 @@ RETCODE		SQL_API
 PGAPI_MoreResults(
 				  HSTMT hstmt)
 {
-	const char *func = "PGAPI_MoreResults";
+	CSTR func = "PGAPI_MoreResults";
 	StatementClass	*stmt = (StatementClass *) hstmt;
 	QResultClass	*res;
 
@@ -1553,6 +1572,8 @@ PGAPI_MoreResults(
 	if (res = SC_get_Curres(stmt), res)
 	{
 		stmt->diag_row_count = res->recent_processed_row_count;
+		stmt->rowset_start = -1;
+		stmt->currTuple = -1;
 		return SQL_SUCCESS;
 	} 
 	return SQL_NO_DATA_FOUND;
@@ -1667,7 +1688,7 @@ static void UndoRollback(StatementClass *stmt, QResultClass *res)
 		status = keyset[index].status;
 		if (0 != (status & CURS_SELF_ADDING))
 		{
-			ridx = index - stmt->rowset_start + res->base;
+			ridx = GIdx2ResultIdx(index, stmt, res);
 			if (ridx >=0 && ridx < res->num_backend_rows)
 			{
 				TupleField *tuple = res->backend_tuples + res->num_fields * ridx;
@@ -1852,7 +1873,7 @@ SC_pos_reload(StatementClass *stmt, UDWORD global_ridx, UWORD *count, BOOL logCh
 		SC_set_error(stmt, STMT_INVALID_OPTION_IDENTIFIER, "the statement is read-only");
 		return SQL_ERROR;
 	}
-	res_ridx = global_ridx - stmt->rowset_start + res->base;
+	res_ridx = GIdx2ResultIdx(global_ridx, stmt, res);
 	if (!(oid = getOid(res, global_ridx)))
 	{
 		SC_set_error(stmt, STMT_ROW_VERSION_CHANGED, "the row was already deleted ?");
@@ -1941,7 +1962,7 @@ SC_pos_reload_needed(StatementClass *stmt, UDWORD flag)
 		SC_set_error(stmt, STMT_INVALID_OPTION_IDENTIFIER, "the statement is read-only");
 		return SQL_ERROR;
 	}
-	limitrow = stmt->rowset_start + res->rowset_size;
+	limitrow = RowIdx2GIdx(res->rowset_size, stmt);
 	if (limitrow > res->num_total_rows)
 		limitrow = res->num_total_rows;
 	if (create_from_scratch)
@@ -1954,7 +1975,7 @@ SC_pos_reload_needed(StatementClass *stmt, UDWORD flag)
 			if (res->backend_tuples[i].value)
 				free(res->backend_tuples[i].value);
 		}
-		brows = limitrow - stmt->rowset_start;
+		brows = GIdx2RowIdx(limitrow, stmt);
 		if (brows > res->count_backend_allocated)
 		{
 			res->backend_tuples = realloc(res->backend_tuples, sizeof(TupleField) * res->num_fields * brows);
@@ -1998,7 +2019,7 @@ SC_pos_reload_needed(StatementClass *stmt, UDWORD flag)
 					{
 						if (oid == getOid(res, k))
 						{
-							l = k - stmt->rowset_start + res->base;
+							l = GIdx2ResultIdx(k, stmt, res);
 							tuple = res->backend_tuples + res->num_fields * l;
 							tuplew = qres->backend_tuples + qres->num_fields * j;
 							for (m = 0; m < res->num_fields; m++, tuple++, tuplew++)
@@ -2118,7 +2139,7 @@ SC_pos_newload(StatementClass *stmt, UInt4 oid, BOOL tidRef)
 			}
 			KeySetSet(tuplen, qres->num_fields, res->keyset + res->num_total_rows);
 
-			if (res->num_total_rows == res->num_backend_rows - res->base + stmt->rowset_start)
+			if (res->num_total_rows == ResultIdx2GIdx(res->num_backend_rows, stmt, res))
 			{
 				if (res->num_backend_rows >= res->count_backend_allocated)
 				{
@@ -2477,6 +2498,7 @@ irow_insert(RETCODE ret, StatementClass *stmt, StatementClass *istmt, int addpos
 		ARDFields	*opts = SC_get_ARD(stmt);
 		QResultClass	*ires = SC_get_Curres(istmt);
 		const char *cmdstr;
+		BindInfoClass	*bookmark;
 
 		cmdstr = QR_get_command((ires->next ? ires->next : ires));
 		if (cmdstr &&
@@ -2499,15 +2521,20 @@ irow_insert(RETCODE ret, StatementClass *stmt, StatementClass *istmt, int addpos
 				if (SQL_ERROR == qret)
 					return qret;
 			}
-			if (opts->bookmark->buffer)
+			bookmark = opts->bookmark;
+			if (bookmark->buffer)
 			{
 				char	buf[32];
 				UInt4	offset = opts->row_offset_ptr ? *opts->row_offset_ptr : 0;
 
-				sprintf(buf, "%ld", addpos + 1);
-				copy_and_convert_field(stmt, 0, buf,
-                         		SQL_C_ULONG, opts->bookmark->buffer + offset,
-					0, opts->bookmark->used ? opts->bookmark->used
+				sprintf(buf, "%lu", addpos + 1);
+				copy_and_convert_field(stmt,
+					PG_TYPE_INT4,
+					buf,
+                         		bookmark->returntype,
+					bookmark->buffer + offset,
+					bookmark->buflen,
+					bookmark->used ? bookmark->used
 					+ (offset >> 2) : NULL);
 			}
 		}
@@ -2644,7 +2671,7 @@ SC_pos_add(StatementClass *stmt,
 	PGAPI_FreeStmt(hstmt, SQL_DROP);
 	if (SQL_SUCCESS == ret && res->keyset)
 	{
-		int	global_ridx = res->num_total_rows + stmt->rowset_start - res->base - 1;
+		int	global_ridx = res->num_total_rows - 1;
 		if (CC_is_in_trans(conn))
 		{
 
@@ -2731,7 +2758,7 @@ PGAPI_SetPos(
 			 UWORD fOption,
 			 UWORD fLock)
 {
-	static char *func = "PGAPI_SetPos";
+	CSTR func = "PGAPI_SetPos";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) hstmt;
 	ConnectionClass	*conn = SC_get_conn(stmt);
@@ -2740,7 +2767,7 @@ PGAPI_SetPos(
 	UWORD		nrow;
 	ARDFields	*opts;
 	BindInfoClass *bindings;
-	UDWORD		global_ridx;
+	UDWORD		global_ridx, rowsetSize;
 	BOOL		auto_commit_needed = FALSE;
 
 	if (!stmt)
@@ -2771,6 +2798,11 @@ PGAPI_SetPos(
 		return SQL_ERROR;
 	}
 
+#if (ODBCVER >= 0x0300)
+	rowsetSize = (stmt->transition_status == 7 ? opts->size_of_rowset_odbc2 : opts->size_of_rowset);
+#else
+	rowsetSize = opts->size_of_rowset_odbc2;
+#endif /* ODBCVER */
 	if (irow == 0) /* bulk operation */
 	{
 		if (SQL_POSITION == fOption)
@@ -2780,7 +2812,7 @@ PGAPI_SetPos(
 			return SQL_ERROR;
 		}
 		start_row = 0;
-		end_row = opts->rowset_size - 1;
+		end_row = rowsetSize - 1;
 	}
 	else
 	{
@@ -2813,7 +2845,7 @@ PGAPI_SetPos(
 	ridx = -1;
 	for (i = nrow = 0, processed = 0; nrow <= end_row; i++)
 	{
-		global_ridx = i + stmt->rowset_start;
+		global_ridx = RowIdx2GIdx(i, stmt);
 		if (SQL_ADD != fOption)
 		{
 			if ((int) global_ridx >= res->num_total_rows)
@@ -2869,7 +2901,7 @@ PGAPI_SetPos(
 	{
 		if (SQL_ADD != fOption && ridx >= 0) /* for SQLGetData */
 		{ 
-			stmt->currTuple = stmt->rowset_start + ridx;
+			stmt->currTuple = RowIdx2GIdx(ridx, stmt);
 			QR_set_position(res, ridx);
 		}
 	}
@@ -2887,7 +2919,7 @@ PGAPI_SetScrollOptions( HSTMT hstmt,
 				SDWORD crowKeyset,
 				UWORD crowRowset)
 {
-	static char *func = "PGAPI_SetScrollOptions";
+	CSTR func = "PGAPI_SetScrollOptions";
 	StatementClass *stmt = (StatementClass *) hstmt;
 
 	mylog("PGAPI_SetScrollOptions fConcurrency=%d crowKeyset=%d crowRowset=%d\n",
@@ -2906,7 +2938,7 @@ PGAPI_SetCursorName(
 					UCHAR FAR * szCursor,
 					SWORD cbCursor)
 {
-	static char *func = "PGAPI_SetCursorName";
+	CSTR func = "PGAPI_SetCursorName";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	int			len;
 
@@ -2940,7 +2972,7 @@ PGAPI_GetCursorName(
 					SWORD cbCursorMax,
 					SWORD FAR * pcbCursor)
 {
-	static char *func = "PGAPI_GetCursorName";
+	CSTR func = "PGAPI_GetCursorName";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	int			len = 0;
 	RETCODE		result;
