@@ -86,6 +86,8 @@ void
 SOCK_Destructor(SocketClass *self)
 {
 	mylog("SOCK_Destructor\n");
+	if (!self)
+		return;
 	if (self->socket != -1)
 	{
 		SOCK_put_char(self, 'X');
@@ -106,7 +108,15 @@ SOCK_Destructor(SocketClass *self)
 char
 SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname)
 {
-	struct hostent *host;
+#if defined (POSIX_MULTITHREAD_SUPPORT)
+    const int bufsz = 8192; 
+    char buf[bufsz];
+    int error = 0;
+    struct hostent host;
+    struct hostent* hp = &host;
+#else
+    struct hostent* hp;
+#endif 
 	unsigned long iaddr;
 
 	if (self->socket != -1)
@@ -124,18 +134,36 @@ SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname)
 	iaddr = inet_addr(hostname);
 	if (iaddr == INADDR_NONE)
 	{
-		host = gethostbyname(hostname);
-		if (host == NULL)
+#if defined (POSIX_MULTITHREAD_SUPPORT) 
+  #if defined (HAVE_GETIPNODEBYNAME) /* Free-BSD ? */
+	hp = getipnodebyname(hostname, AF_INET, 0, &error); 
+  #elif defined (PGS_REENTRANT_API_1) /* solaris, irix */
+        hp = gethostbyname_r(hostname, hp, buf, bufsz, &error);
+  #elif defined (PGS_REENTRANT_API_2) /* linux */
+        int result = 0;
+        result = gethostbyname_r(hostname, hp, buf, bufsz, &hp, &error);
+        if (result)
+          hp = 0;
+  #else
+        hp = gethostbyname(hostname);
+  #endif
+#else
+        hp = gethostbyname(hostname);
+#endif
+		if (hp == NULL)
 		{
 			self->errornumber = SOCKET_HOST_NOT_FOUND;
 			self->errormsg = "Could not resolve hostname.";
 			return 0;
 		}
-		memcpy(&(self->sadr.sin_addr), host->h_addr, host->h_length);
+		memcpy(&(self->sadr.sin_addr), hp->h_addr, hp->h_length);
 	}
 	else
 		memcpy(&(self->sadr.sin_addr), (struct in_addr *) & iaddr, sizeof(iaddr));
 
+#if defined (HAVE_GETIPNODEBYNAME)
+	freehostent(hp);
+#endif /* HAVE_GETIPNODEBYNAME */
 	self->sadr.sin_family = AF_INET;
 	self->sadr.sin_port = htons(port);
 
@@ -165,6 +193,8 @@ SOCK_get_n_char(SocketClass *self, char *buffer, int len)
 {
 	int			lf;
 
+	if (!self)
+		return;
 	if (!buffer)
 	{
 		self->errornumber = SOCKET_NULLPOINTER_PARAMETER;
@@ -182,6 +212,8 @@ SOCK_put_n_char(SocketClass *self, char *buffer, int len)
 {
 	int			lf;
 
+	if (!self)
+		return;
 	if (!buffer)
 	{
 		self->errornumber = SOCKET_NULLPOINTER_PARAMETER;
@@ -229,6 +261,8 @@ SOCK_put_string(SocketClass *self, char *string)
 int
 SOCK_get_int(SocketClass *self, short len)
 {
+	if (!self)
+		return 0;
 	switch (len)
 	{
 		case 2:
@@ -266,6 +300,8 @@ SOCK_put_int(SocketClass *self, int value, short len)
 {
 	unsigned int rv;
 
+	if (!self)
+		return;
 	switch (len)
 	{
 		case 2:
@@ -291,6 +327,8 @@ SOCK_flush_output(SocketClass *self)
 {
 	int			written;
 
+	if (!self)
+		return;
 	written = send(self->socket, (char *) self->buffer_out, self->buffer_filled_out, 0);
 	if (written != self->buffer_filled_out)
 	{
@@ -304,6 +342,8 @@ SOCK_flush_output(SocketClass *self)
 unsigned char
 SOCK_get_next_byte(SocketClass *self)
 {
+	if (!self)
+		return 0;
 	if (self->buffer_read_in >= self->buffer_filled_in)
 	{
 		/*
@@ -338,6 +378,8 @@ SOCK_put_next_byte(SocketClass *self, unsigned char next_byte)
 {
 	int			bytes_sent;
 
+	if (!self)
+		return;
 	self->buffer_out[self->buffer_filled_out++] = next_byte;
 
 	if (self->buffer_filled_out == self->buffer_size)
