@@ -81,7 +81,7 @@ RETCODE		SQL_API
 PGAPI_AllocStmt(HDBC hdbc,
 				HSTMT FAR * phstmt)
 {
-	static char *func = "PGAPI_AllocStmt";
+	CSTR func = "PGAPI_AllocStmt";
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
 	StatementClass *stmt;
 
@@ -135,7 +135,7 @@ RETCODE		SQL_API
 PGAPI_FreeStmt(HSTMT hstmt,
 			   UWORD fOption)
 {
-	static char *func = "PGAPI_FreeStmt";
+	CSTR func = "PGAPI_FreeStmt";
 	StatementClass *stmt = (StatementClass *) hstmt;
 
 	mylog("%s: entering...hstmt=%u, fOption=%d\n", func, hstmt, fOption);
@@ -183,6 +183,7 @@ PGAPI_FreeStmt(HSTMT hstmt,
 		 * this should discard all the results, but leave the statement
 		 * itself in place (it can be executed again)
 		 */
+		stmt->transition_status = 0;
 		if (!SC_recycle_statement(stmt))
 		{
 			/* errormsg passed in above */
@@ -217,6 +218,9 @@ InitializeStatementOptions(StatementOptions *opt)
 	opt->cursor_type = SQL_CURSOR_FORWARD_ONLY;
 	opt->retrieve_data = SQL_RD_ON;
 	opt->use_bookmarks = SQL_UB_OFF;
+#if (ODBCVER >= 0x0300)
+	opt->metadata_id = SQL_FALSE;
+#endif /* ODBCVER */
 }
 
 
@@ -227,8 +231,11 @@ void
 InitializeARDFields(ARDFields *opt)
 {
 	memset(opt, 0, sizeof(ARDFields));
-	opt->rowset_size = 1;
+#if (ODBCVER >= 0x0300)
+	opt->size_of_rowset = 1;
+#endif /* ODBCVER */
 	opt->bind_size = 0;		/* default is to bind by column */
+	opt->size_of_rowset_odbc2 = 1;
 }
 /*
  * APDFields initialize
@@ -259,6 +266,7 @@ SC_Constructor(void)
 		rv->prepared = FALSE;
 		rv->status = STMT_ALLOCATED;
 		rv->internal = FALSE;
+		rv->transition_status = 0;
 
 		rv->__error_message = NULL;
 		rv->__error_number = 0;
@@ -840,7 +848,7 @@ SC_get_bookmark(StatementClass *self)
 RETCODE
 SC_fetch(StatementClass *self)
 {
-	static char *func = "SC_fetch";
+	CSTR func = "SC_fetch";
 	QResultClass *res = SC_get_Curres(self);
 	ARDFields	*opts;
 	int			retval,
@@ -958,9 +966,7 @@ SC_fetch(StatementClass *self)
 				value = QR_get_value_backend(res, lf);
 			else
 			{
-				int curt = res->base;
-				if (self->rowset_start >= 0)
-					curt += (self->currTuple - self->rowset_start);
+				int	curt = GIdx2ResultIdx(self->currTuple, self, res);
 				value = QR_get_value_backend_row(res, curt, lf);
 			}
 
@@ -1021,10 +1027,15 @@ SC_fetch(StatementClass *self)
 RETCODE
 SC_execute(StatementClass *self)
 {
-	static char *func = "SC_execute";
+	CSTR func = "SC_execute";
 	ConnectionClass *conn;
 	APDFields	*apdopts;
-	char		was_ok, was_nonfatal;
+	char		was_ok, was_nonfatal, was_rows_affected = 1;
+	/* was_rows_affected is set to 0 iff an UPDATE or DELETE affects 
+	 * no rows. In this instance the driver should return
+	 * SQL_NO_DATA_FOUND instead of SQL_SUCCESS.
+	 */
+ 
 	QResultClass	*res = NULL;
 	Int2		oldstatus,
 				numcols;
@@ -1047,6 +1058,15 @@ SC_execute(StatementClass *self)
 	 * 2) we are in autocommit off state and the statement isn't of type
 	 * OTHER.
 	 */
+	ENTER_CONN_CS(conn);
+	if (CONN_EXECUTING == conn->status)
+	{
+		LEAVE_CONN_CS(conn);
+		SC_set_error(self, STMT_SEQUENCE_ERROR, "Connection is already in use.");
+		SC_log_error(func, "", self);
+		mylog("%s: problem with connection\n", func);
+		return SQL_ERROR;
+	}
 	is_in_trans = CC_is_in_trans(conn);
 	if (!self->internal && !is_in_trans &&
 		(SC_is_fetchcursor(self) ||
@@ -1058,6 +1078,7 @@ SC_execute(StatementClass *self)
 			qflag |= GO_INTO_TRANSACTION;
                 else if (!CC_begin(conn))
                 {
+			LEAVE_CONN_CS(conn);
 			SC_set_error(self, STMT_EXEC_ERROR, "Could not begin a transaction");
                         SC_log_error(func, "", self);
                         return SQL_ERROR;
@@ -1138,12 +1159,20 @@ SC_execute(StatementClass *self)
 	if (CONN_DOWN != conn->status)
 		conn->status = oldstatus;
 	self->status = STMT_FINISHED;
+	LEAVE_CONN_CS(conn);
 
 	/* Check the status of the result */
 	if (res)
 	{
 		was_ok = QR_command_successful(res);
 		was_nonfatal = QR_command_nonfatal(res);
+		if (res->command &&
+			(strncmp(res->command, "UPDATE", 6) == 0 ||
+			strncmp(res->command, "DELETE", 6) == 0) &&
+			strtoul(res->command + 7, NULL, 0) == 0)
+		{
+			was_rows_affected = 0;
+		}
 
 		if (was_ok)
 			SC_set_errornumber(self, STMT_OK);
@@ -1246,7 +1275,10 @@ SC_execute(StatementClass *self)
 		}
 	}
 	if (SC_get_errornumber(self) == STMT_OK)
-		return SQL_SUCCESS;
+		if (was_rows_affected)
+			return SQL_SUCCESS;
+		else
+			return SQL_NO_DATA_FOUND;
 	else if (SC_get_errornumber(self) == STMT_INFO_ONLY)
 		return SQL_SUCCESS_WITH_INFO;
 	else
@@ -1270,7 +1302,13 @@ SC_log_error(const char *func, const char *desc, const StatementClass *self)
 		QResultClass *res = SC_get_Result(self);
 		const ARDFields	*opts = SC_get_ARD(self);
 		const APDFields	*apdopts = SC_get_APD(self);
+		int	rowsetSize;
 
+#if (ODBCVER >= 0x0300)
+		rowsetSize = (7 == self->transition_status ? opts->size_of_rowset_odbc2 : opts->size_of_rowset);
+#else
+		rowsetSize = opts->size_of_rowset_odbc2;
+#endif /* ODBCVER */
 		qlog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
 		mylog("STATEMENT ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
 		qlog("                 ------------------------------------------------------------\n");
@@ -1282,7 +1320,7 @@ SC_log_error(const char *func, const char *desc, const StatementClass *self)
 		qlog("                 stmt_with_params='%s'\n", nullcheck(self->stmt_with_params));
 		qlog("                 data_at_exec=%d, current_exec_param=%d, put_data=%d\n", self->data_at_exec, self->current_exec_param, self->put_data);
 		qlog("                 currTuple=%d, current_col=%d, lobj_fd=%d\n", self->currTuple, self->current_col, self->lobj_fd);
-		qlog("                 maxRows=%d, rowset_size=%d, keyset_size=%d, cursor_type=%d, scroll_concurrency=%d\n", self->options.maxRows, opts->rowset_size, self->options.keyset_size, self->options.cursor_type, self->options.scroll_concurrency);
+		qlog("                 maxRows=%d, rowset_size=%d, keyset_size=%d, cursor_type=%d, scroll_concurrency=%d\n", self->options.maxRows, rowsetSize, self->options.keyset_size, self->options.cursor_type, self->options.scroll_concurrency);
 		qlog("                 cursor_name='%s'\n", nullcheck(self->cursor_name));
 
 		qlog("                 ----------------QResult Info -------------------------------\n");
