@@ -114,7 +114,7 @@ enum
 	STMT_PARSE_NONE = 0,
 	STMT_PARSE_COMPLETE,
 	STMT_PARSE_INCOMPLETE,
-	STMT_PARSE_FATAL,
+	STMT_PARSE_FATAL
 };
 
 /*	Result style */
@@ -122,7 +122,7 @@ enum
 {
 	STMT_FETCH_NONE = 0,
 	STMT_FETCH_NORMAL,
-	STMT_FETCH_EXTENDED,
+	STMT_FETCH_EXTENDED
 };
 
 
@@ -182,7 +182,7 @@ struct StatementClass_
 						 * prepared at the server ? */
 	char		internal;		/* Is this statement being called
 								 * internally? */
-
+	char		transition_status;	/* Transition status */
 	char		cursor_name[MAX_CURSOR_LEN + 1];
 
 	char	   *stmt_with_params;		/* statement after parameter
@@ -227,6 +227,11 @@ struct StatementClass_
 #define	SC_get_errornumber(a) (a->__error_number)
 #define	SC_set_errornumber(a, n) (a->__error_number = n)
 #define	SC_get_errormsg(a) (a->__error_message)
+#if (ODBCVER >= 0x0300)
+#define	SC_is_lower_case(a, b) (a->options.metadata_id || b->connInfo.lower_case_identifier)
+#else
+#define	SC_is_lower_case(a, b) (b->connInfo.lower_case_identifier)
+#endif /* ODBCVER */
 
 /*	options for SC_free_params() */
 #define STMT_FREE_PARAMS_ALL				0
@@ -244,7 +249,7 @@ struct StatementClass_
 #define SC_is_prepare_before_exec(a)	((a->miscinfo & 4L) != 0)
 
 /* For Multi-thread */
-#if defined(WIN_FREETHREAD_SUPPORT)
+#if defined(WIN_MULTITHREAD_SUPPORT)
 #define INIT_STMT_CS(x)		InitializeCriticalSection(&((x)->cs))
 #define ENTER_STMT_CS(x)	EnterCriticalSection(&((x)->cs))
 #define LEAVE_STMT_CS(x)	LeaveCriticalSection(&((x)->cs))
@@ -290,4 +295,15 @@ RETCODE		SC_pos_delete(StatementClass *self, UWORD irow, UDWORD index);
 RETCODE		SC_pos_refresh(StatementClass *self, UWORD irow, UDWORD index);
 RETCODE		SC_pos_add(StatementClass *self, UWORD irow);
 
+/*
+ *	Macros to convert global index <-> relative index in resultset/rowset
+ */
+/* a global index to the relative index in a rowset */
+#define	GIdx2RowIdx(gidx, stmt)	(gidx - stmt->rowset_start)
+/* a global index to the relative index in a resultset(not a rowset) */
+#define	GIdx2ResultIdx(gidx, stmt, res)	((stmt->rowset_start < 0) ? res->base : gidx - stmt->rowset_start + res->base)
+/* a relative index in a rowset to the global index */
+#define	RowIdx2GIdx(ridx, stmt)	(ridx + stmt->rowset_start)
+/* a relative index in a resultset to the global index */
+#define	ResultIdx2GIdx(ridx, stmt, res)	(ridx - res->base + stmt->rowset_start)
 #endif
